import os
os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide"
from os import path
from pygame import mixer
from twodxPlay.twodx import Twodx
import sys, traceback
from datetime import timedelta
import struct


if len(sys.argv) <= 1:
    print("2dxPlay V1.0.0")
    print("Usage: 2dxplay infile.2dx")
    sys.exit(1)

infile = sys.argv[1]

mixer.init()

try:
    twodx_file = Twodx(path.abspath(infile))
except struct.error:
    traceback.print_exc()
    print("Probably not a 2dx file.")
    sys.exit(1)
except:
    traceback.print_exc()
    sys.exit(1)

current_track = 1

def main():
    global current_track
    try:
        while current_track-1 < twodx_file.file_count:
            wav = twodx_file.load(current_track-1)
            sound = mixer.Sound(wav)
            channel = sound.play()
            length = timedelta(seconds=int(sound.get_length()))
            print(f"Playing: {infile}    Track {current_track}/{twodx_file.file_count}    Length: {length}")
            while channel.get_busy():
                pass
            current_track += 1
    except KeyboardInterrupt:
        mixer.stop()
        while "the answer is invalid":
            try:
                reply = str(input('Enter track number to jump to, or q to quit: ')).lower().strip()
            except (EOFError, KeyboardInterrupt):
                sys.exit(0)

            if reply == 'q':
                sys.exit(0)
            if reply.isdigit() and int(reply) in range(1, twodx_file.file_count + 1):
                current_track = int(reply)
                main()

    except Exception:
        traceback.print_exc()
        sys.exit(1)

if __name__ == "__main__":
    main()



import struct
from io import BytesIO


# 2dx file structure ported from https://github.com/mon/2dxTools
class Twodx:

    def __init__(self, path):
        self.path = path
        try:
            with open(self.path, 'rb') as f:
                self.name = f.read(16)
                self.header_size = struct.unpack('I', f.read(4))[0]
                self.file_count = struct.unpack('I', f.read(4))[0]
                f.seek(48, 1)
                self.file_offsets = [struct.unpack('I', f.read(4))[0] for i in range(self.file_count)]
        except:
            raise

    def load(self, file: int):
        with open(self.path, 'rb') as f:
            f.seek(self.file_offsets[file])
            dx = f.read(4)
            header_size = struct.unpack('I', f.read(4))[0]  # Should be "2DX9";
            wav_size = struct.unpack('I', f.read(4))[0]  # Always 24 bytes, includes dx chars
            unk1 = struct.unpack('h', f.read(2))[0]  # Always 0x3231
            track_id = struct.unpack('h', f.read(2))[0]  # Always -1 for previews, 0-7 for song + effected versions, 9 to 11 used for a few effects
            unk2 = struct.unpack('h', f.read(2))[0]  # All 64, except song selection change 'click' is 40
            attenuation = struct.unpack('h', f.read(2))[0]  #  0-127 for varying quietness
            loop_point = struct.unpack('I', f.read(4))[0]  #  Sample to loop at * 4

            f.seek(self.file_offsets[file] + header_size)
            wav = BytesIO(f.read(wav_size))
            return wav



import os
from distutils.core import setup
import setuptools

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='2dxPlay',
    version='1.0.0',
    packages=['twodxPlay'],
    entry_points='''
        [console_scripts]
        2dxplay=twodxPlay.main:main
    ''',
    keywords=['2dx'],
    url='https://github.com/camprevail/2dxPlay',
    license='MIT',
    author='camprevail',
    author_email='cam.anderson573@gmail.com',
    description='Play a .2dx file.',
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["pygame", "setuptools"],
)
